@make(ais)
@aiInfo(topic=<Pragma ELABORATE for bodiless packages with tasks>,
number=[AI-00236/12],
classcode=[BI],
statuscode=[WJ],
mainref=[10.05@\(04)],
date=[88-12-14])
@aiHeading()
@begin(history)
@begin(toplines)
@mainref()
@standard[B@\(03)]
!class binding interpretation 85-05-17
!status approved by WG9/AJPO 86-07-22
@end(toplines)
!status approved by Director, AJPO 86-07-22
!status approved by WG9/Ada Board 86-07-22
!status approved by Ada Board 86-07-22
!status approved by WG9 86-05-09
!status committee-approved (9-1-2) 85-11-20 (by ballot)
!status committee-approved (8-1-0) 85-09-04 (pending letter ballot)
!status failed letter ballot (5-6-1) 85-09-04
!status committee-approved (9-0-2) 85-05-17
!status work-item 85-02-04
!status received 84-04-13
@references[!references @CommentNum[83-00350]83-00350, @CommentNum[83-00591]83-00591, @CommentNum[83-00617]83-00617, @CommentNum[83-00660]83-00660, @CommentNum[83-00672]83-00672]
@topic{!topic @value(aitopic)}
@end(history)

@subheading[!summary 85-07-25]

If a package declares a task object but no package body is required or
provided by a programmer, 9.3(5) says an implicit body is provided.  The
effect of a pragma ELABORATE that names such a package is to require that the
implicitly provided package body be elaborated, thereby activating the task
declared in the package.

@subheading[!question 85-09-14]

Consider this example:
@begin(example)
@key[with] TASK_@|TYPES;
@key[pragma] ELABORATE (TASK_@|TYPES);   -- to prevent PROGRAM_@|ERROR
@key[package] P @key[is]
     TSK : TASK_@|TYPES.SOME_@|TASK_@|TYPE;
@key[end] P;
@hinge()

@key[with] P;
@key[pragma] ELABORATE (P);    -- requires elaboration of P's implicit body
@key[package] Q @key[is] ...
@end(example)
Assume that P does not later have a body supplied.  10.5(4) says P @dq{must have
a library unit body}.  Since P has no body, this would seem to imply that the
pragma ELABORATE should be ignored [2.8(9)].  However, 9.3(5) says there is an
implicit package body for P that causes activation of the task declared within
P.  Thus the above example might be intended to insure that the task TSK is
activated before Q's elaboration.  Is the implicit body elaborated?

!wording 84-04-13


@subheading[!recommendation 85-07-25]

If a pragma ELABORATE names a package that has an implicitly provided body,
the implicit body is elaborated before the compilation unit associated with
the pragma.

@subheading[!discussion 85-09-14]

9.3(5) says, regarding the rules for activating tasks:
@begin(quotation)
For the above rules, @ellipsis()  for any package without a  package
body, an  implicit package  body  containing a  single  null
statement is assumed.
@end(quotation)
10.5(4) says, with respect to the pragma ELABORATE:
@begin(quotation)
Each argument of  such a pragma  must be the  simple name of  a
library unit mentioned by the context clause, and this  library
unit must have a library unit body.
@end(quotation)
The basic question is: can the pragma ELABORATE be obeyed for a package that
declares a task object but that has no explicitly provided package body?

It considering this question, it should be noted that 10.5(4) uses the term
@dq{library unit body}, which, by 1.5(6) is a reference to the syntactic term,
library_@|unit_@|body.  This reference to a syntactic term does not imply the
body must be provided explicitly by a programmer, since the rules of the
Standard apply equally well to implicitly and explicitly declared units.  For
example, 10.5(1-2) specifies that before execution of the main program all
library unit bodies must be elaborated in a certain order.  These rules also
use the term @dq{library unit body,} and here, the intent is clearly to cover
the bodies implicitly declared by 9.3(5) as well as those explicitly declared
by a programmer.

Also, although 9.3(5) says @dq{For the above rules} and thereby seems to limit
the effect of the paragraph in some way that might be understood to exclude
rules given in 10.5, such an interpretation is untenable, since the
elaboration order rules given in 10.5(1-2) must be considered to apply to
implicit package bodies created by 9.3(5).

In short, it would be unreasonable to interpret 10.5(1-2) as applying to
implicit as well as explcit package bodies, and then to interpret identical
phrases in 10.5(4) to apply only to explicitly declared package bodies.

In addition, it is important that the pragma be allowed to apply to
implicitly generated bodies since a programmer cannot always know when a task
object is being created.  When an object having a limited private type is
declared in a package, a task can be declared without the programmer's
knowledge.  It would be a nuisance (to say the least) to have to provide an
explicit dummy package body just to ensure that any potential task will be
activated by a pragma ELABORATE.

Finally, it is always harmless to name a package that has no (explicit or
implicit) body, since the pragma will then be ignored [2.8(9)]; at most, an
implementation will produce a warning message.

In short, when the  pragma ELABORATE names a  package that has an  implicitly
provided body, it is both useful and consistent with the Standard to  require
that the implicit body be  elaborated before the compilation unit  containing
the pragma.

